local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")
local faction = UnitFactionGroup("player")

local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

local AceGUI = LibStub("AceGUI-3.0")
do
	local Type = "Confirmless_EditBox"
	local Version = 1

	-- Cloned in place because doing the override went nuts in 2nd OnShow
	-- source [libs\AceGUI-3.0\widgets\AceGUIWidget-EditBox.lua]
	local function Base_OnAcquire(self)
		self:SetHeight(26)
		self:SetWidth(200)
		self:SetDisabled(false)
		self:SetLabel()
		self.showbutton = true
	end
	-- Cloned in place
	-- source [libs\AceGUI-3.0\widgets\AceGUIWidget-EditBox.lua]
	local function Base_EditBox_OnTextChanged(this)
		local self = this.obj
		local value = this:GetText()
		if tostring(value) ~= tostring(self.lasttext) then
			self:Fire("OnTextChanged",value)
			self.lasttext = value
--			ShowButton(self)
		end
	end

	local function Derived_OnAcquire(self)
		Base_OnAcquire(self)
		self.showbutton = false
	end
	local function Derived_EditBox_OnTextChanged(this)
		Base_EditBox_OnTextChanged(this)
		local user = this.obj:GetUserDataTable()

		-- NOTE: This only works if 'set' is a function rather than a func name.
		user.option.set(user, this.obj.editbox:GetText())
	end

	-- Based on SetText
	-- source [libs\AceGUI-3.0\widgets\AceGUIWidget-EditBox.lua]
	local function Base_SetText(self, text)
		self.lasttext = text or ""
		self.editbox:SetText(text or "")
		self.editbox:SetCursorPosition(0)
	end
	local function Derived_SetText(self, text)
		local len = strlen(text)
		self.lasttext = text or ""
		self.editbox:SetText(text or "")
		self.editbox:SetCursorPosition(len)
		self.editbox:HighlightText(len, len)
		self.SetText = Base_SetText
	end

	-- Based on EditBox_OnEscapePressed
	-- source [libs\AceGUI-3.0\widgets\AceGUIWidget-EditBox.lua]
	local function Base_OnEscapePressed(this)
		this:ClearFocus()
	end
	local function Derived_OnEscapePressed(this)
		local textOld = this:GetText()
		local len = strlen(textOld or "")
		if (len > 0) then
			this:SetText("")
		else
			this:ClearFocus()
		end
	end

	local function Constructor()
		local self = AceGUI:Create("EditBox")
		self.OnAcquire = Derived_OnAcquire
		self.editbox:SetScript("OnTextChanged", Derived_EditBox_OnTextChanged)
		self.editbox:SetScript("OnEnterPressed", function() end)
		if (Capitalist.inclusiveTerm) then
			self.editbox:SetScript("OnShow", function()
													if (Capitalist.inclusiveTerm ~= nil) then
														local len = strlen(self.editbox:GetText())
														self.editbox:SetFocus()
														self.editbox:SetCursorPosition(len)
														self.editbox:HighlightText(len, len)
													end
											end
									)
			
			if (Capitalist.inclusiveTerm ~= nil) then
				self.SetText = Derived_SetText
			end
		end
		self.editbox:SetScript("OnEscapePressed", Derived_OnEscapePressed)
		return self
	end
	
	AceGUI:RegisterWidgetType(Type, Constructor, Version)
end

function Capitalist:BuildOptions()
	self:DebugEntry("BuildOptions")

	self.options = {
		type = "group",
		args = {
			general = {
				name = L["General"],
				type = "group",
				order = 1,
				args = {
					overview = {
						name = L["General Overview"],
						type = "description",
						order = 0,
					},
					minProfit = {
						name = L["Min Profit"],
						order = 1,
						desc = L["Min Profit Tooltip"],
						type = "input",
						width = "half",
						pattern = "^[0-9]*\.?[0-9]+$",
						set = function(info, val)
								self.db.profile.minProfit = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minProfit / 10000
							  end,
					},
					margin = {
						name = L["Price Margin"],
						order = 2,
						desc = L["Price Margin Tooltip"],
						type = "input",
						width = "half",
						pattern = "^[0-9]*\.?[0-9]+$",
						set = function(info, val)
								self.db.profile.margin = tonumber(val) or 0
							  end,
						get = function(info)
								return ""..self.db.profile.margin
							  end,
					},
					minMargin = {
						name = L["Min Increase"],
						order = 3,
						desc = L["Min Increase Tooltip"],
						type = "input",
						width = "half",
						pattern = "^[0-9]*\.?[0-9]+$",
						set = function(info, val)
								self.db.profile.minMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minMargin / 10000
							  end,
					},
					maxMargin = {
						name = L["Max Increase"],
						order = 4,
						desc = L["Max Increase Tooltip"],
						type = "input",
						width = "half",
						pattern = "^[0-9]*\.?[0-9]+$",
						set = function(info, val)
								self.db.profile.maxMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.maxMargin / 10000
							  end,
					},
					showByDefault = {
						name = L["Show by Default"],
						order = 5,
						desc = L["Show by Default Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.showByDefault = val
							  end,
						get = function(info)
								return self.db.profile.showByDefault
							  end,
					},
					bidAtCost = {
						name = L["Bid Price at Cost"],
						order = 6,
						desc = L["Bid Price at Cost Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.enableBidAtCost = true
								else
									self.db.profile.enableBidAtCost = nil
								end
							  end,
						get = function(info)
								return self.db.profile.enableBidAtCost
							  end,
					},
					merchantCosting = {
						name = L["Merchant Costing"],
						order = 7,
						desc = L["Merchant Costing Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.enableMerchantCosting = true
									if (self.db.profile.enableMerchantValuation) then
										self:CalcValues()
									end
								else
									self.db.profile.enableMerchantCosting = nil
									self:ClearValues()
								end
							  end,
						get = function(info)
								return self.db.profile.enableMerchantCosting
							  end,
					},
					merchantValuation = {
						name = L["Merchant Valuation"],
						order = 8,
						desc = function()
									if (self.db.profile.enableMerchantCosting) then
										return L["Merchant Valuation Tooltip Green"]
									else
										return L["Merchant Valuation Tooltip Red"]
									end
								end,
						type = "toggle",
						tristate = true,
						set = function(info, val)
								if (val) then
									self.db.profile.enableMerchantValuation = true
									if (self.db.profile.enableMerchantCosting) then
										self:CalcValues()
									end
								else
									self.db.profile.enableMerchantValuation = nil
									self:ClearValues()
								end
							  end,
						get = function(info)
								if (self.db.profile.enableMerchantCosting) then
									return (self.db.profile.enableMerchantValuation == true)
								else
									if (self.db.profile.enableMerchantValuation) then
										return nil
									else
										return false
									end
								end
							  end,
					},
					salvageTracking = {
						name = L["Salvage Tracking"],
						order = 9,
						desc = L["Salvage Tracking Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.enableSalvageTracking = true
								else
									self.db.profile.enableSalvageTracking = nil
								end
							  end,
						get = function(info)
								return self.db.profile.enableSalvageTracking
							  end,
					},
					salvageSupply = {
						name = L["Salvage Supply"],
						order = 10,
						desc = function()
									if (self.db.profile.enableSalvageTracking) then
										return L["Salvage Supply Tooltip Green"]
									else
										return L["Salvage Supply Tooltip Red"]
									end
								end,
						type = "toggle",
						tristate = true,
						set = function(info, val)
								if (val) then
									self.db.profile.enableSalvageSupply = true
								else
									self.db.profile.enableSalvageSupply = nil
								end
							  end,
						get = function(info)
								if (self.db.profile.enableSalvageTracking) then
									return (self.db.profile.enableSalvageSupply == true)
								else
									if (self.db.profile.enableSalvageSupply) then
										return nil
									else
										return false
									end
								end
							  end,
					},
					headerExpansionFilters = {
						name = "Expansion Filters",
						order = 20,
						type = "header",
					},
					Cataclysm = {
						name = L["Cataclysm"],
						order = 21,
						desc = L["Cataclysm Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Cataclysm = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Cataclysm
							  end,
					},
					Northrend = {
						name = L["Northrend"],
						order = 21,
						desc = L["Northrend Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Northrend = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Northrend
							  end,
					},
					BurningCrusade = {
						name = L["Burning Crusade"],
						order = 22,
						desc = L["Burning Crusade Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.BurningCrusade = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.BurningCrusade
							  end,
					},
					Classic = {
						name = L["Classic"],
						order = 23,
						desc = L["Classic Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Classic = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Classic
							  end,
					},
				}
			},
			specific = {
				name = L["Specific"],
				type = "group",
				order = 2,
				args = {
					overview = {
						name = string.format(L["Specific Overview"], character),
						type = "description",
						order = 0,
					},
					crossScan = {
						name = L["Cross Scan"],
						order = 1,
						desc = L["Cross Scan Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.optionsSpecific[realm][character].crossScan = true
								else
									self.db.profile.optionsSpecific[realm][character].crossScan = nil
								end
								self:ApplyFilters()
							end,
						get = function(info)
								return self.db.profile.optionsSpecific[realm][character].crossScan
							end,
					},
					crossPost = {
						name = L["Cross Post"],
						order = 2,
						desc = function()
									if (self.db.profile.optionsSpecific[realm][character].crossScan) then
										return L["Cross Post Tooltip Green"]
									else
										return L["Cross Post Tooltip Red"]
									end
								end,
						type = "toggle",
						tristate = true;
						set = function(info, val)
								if (val) then
									self.db.profile.optionsSpecific[realm][character].crossPost = true
								else
									self.db.profile.optionsSpecific[realm][character].crossPost = nil
								end
								self:ApplyFilters()
							end,
						get = function(info)
								if (self.db.profile.optionsSpecific[realm][character].crossScan) then
									return (self.db.profile.optionsSpecific[realm][character].crossPost == true)
								else
									if (self.db.profile.optionsSpecific[realm][character].crossPost) then
										return nil
									else
										return false
									end
								end
							end,
					},
					crossShop = {
						name = L["Cross Shop"],
						order = 3,
						desc = L["Cross Shop Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.optionsSpecific[realm][character].crossShop = true
								else
									self.db.profile.optionsSpecific[realm][character].crossShop = nil
								end
								self:ApplyFilters()
							end,
						get = function(info)
								return self.db.profile.optionsSpecific[realm][character].crossShop
							end,
					},
					headerPricing = {
						name = "Pricing",
						order = 10,
						type = "header",
					},
					liquidate = {
						name = L["Liquidate"],
						order = 11,
						desc = L["Liquidate Tooltip"],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.optionsSpecific[realm][character].liquidate = true
								else
									self.db.profile.optionsSpecific[realm][character].liquidate = nil
								end
								self:ApplyFilters()
							end,
						get = function(info)
								return self.db.profile.optionsSpecific[realm][character].liquidate
							end,
					},
				}
			},
			rareStatistics = {
				name = L["Rare Statistics"],
				type = "group",
				order = 81,
				args = {}
			},
			salvageStatistics = {
				name = L["Salvage Statistics"],
				type = "group",
				order = 82,
				args = {}
			},
			resourceCosts = {
				name = L["Resource Costs"],
				type = "group",
				order = 83,
				args = {}
			},
			createdProducts = {
				name = L["Created Products"],
				type = "group",
				order = 84,
				args = {}
			},
			usedSkills = {
				name = L["Used Skills"],
				type = "group",
				order = 85,
				args = {}
			},
			notifications = {
				name = L["Notifications"],
				type = "group",
				order = 91,
				args = {
					merchantCostChanges = {
						name = L["Merchant cost changes"],
						order = 1,
						desc = L["Merchant cost changes Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.NotifyMerchantCostChanges = val
							end,
						get = function(info)
								return self.db.profile.filters.global.NotifyMerchantCostChanges
							end,
					},
					merchantValuationChanges = {
						name = L["Merchant valuation changes"],
						order = 2,
						desc = L["Merchant valuation changes Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.NotifyMerchantValuationChanges = val
							end,
						get = function(info)
								return self.db.profile.filters.global.NotifyMerchantValuationChanges
							end,
					},
					salvageTrackingChanges = {
						name = L["Salvage tracking changes"],
						order = 3,
						desc = L["Salvage tracking changes Tooltip"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.NotifySalvageTrackingChanges = val
							end,
						get = function(info)
								return self.db.profile.filters.global.NotifySalvageTrackingChanges
							end,
					},
				},
			},
			data = {
				name = L["Data Management"],
				type = "group",
				order = 92,
				args = {
					overview = {
						name = L["Data Management Overview"],
						type = "description",
						order = 1,
						width = "full",
					},
					dividerCalculations = {
						name = L["Calculations"],
						type = "header",
						order = 10,
						width = "full",
					},
					recalcValues = {
						name = L["Recalc values"],
						order = 11,
						desc = L["Recalc values Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									self:ClearValues()
									self:CalcValues()
								end,
					},
					dividerScanning = {
						name = L["Scanning"],
						type = "header",
						order = 20,
					},
					clearMerchants = {
						name = L["Clear merchant data"],
						order = 21,
						desc = L["Clear merchant data Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											if (self:ResourceGetMerchant(componentName)) then
												self:DebugMisc(componentName)
												self:ResourceSetMerchant(componentName, nil)
												self:SetResourceCopperBuyPrice(componentName, nil)
											end
										end
									end
								end,
					},
					clearProfessions = {
						name = L["Clear profession data"],
						order = 21,
						desc = L["Clear profession data Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									if (self.db.profile.professions) then
										for profession, professionData in pairs(self.db.profile.professions) do
											self.options.args[profession] = nil
										end
									end
									self.db.profile.professions = { }
								end,
					},
					dividerSettings = {
						name = L["Settings"],
						type = "header",
						order = 30,
					},
					clearBuyPrices = {
						name = L["Clear buy prices"],
						order = 31,
						desc = L["Clear buy prices Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											if (not self:ResourceGetMerchant(componentName)) then
												self:DebugMisc(componentName)
												self:SetResourceCopperBuyPrice(componentName, nil)
											end
										end
									end
								end,
					},
					clearCosts = {
						name = L["Clear costs"],
						order = 32,
						desc = L["Clear costs Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											self:DebugMisc(componentName)
											self:SetResourceCopperCost(componentName, nil)
										end
									end
								end,
					},
					dividerStatistics = {
						name = L["Statistics"],
						type = "header",
						order = 40,
					},
					clearCreatedProducts = {
						name = L["Clear created products"],
						order = 41,
						desc = L["Clear created products Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									-- TODO: Update this to not clear preserve salvaging data
									local oRealm = self:GetProfile_Realm_Faction(realm, faction)
									if (oRealm) then
										for crafter, characterData in pairs(oRealm) do
											for profession, professionData in pairs(characterData) do
												for skillName, skillData in pairs(professionData) do
													skillData.numProduct = nil
												end
											end
										end
										self:UpdateStatistics()
									end
								end,
					},
					clearUsedSkills = {
						name = L["Clear used skills"],
						order = 41,
						desc = L["Clear used skills Tooltip"],
						type = "execute",
						width = "full",
						func = function()
									-- TODO: Update this to not clear preserve salvaging data
									local oRealm = self:GetProfile_Realm_Faction(realm, faction)
									if (oRealm) then
										for crafter, characterData in pairs(oRealm) do
											for profession, professionData in pairs(characterData) do
												for skillName, skillData in pairs(professionData) do
													skillData.numCraft = nil
												end
											end
										end
										self:UpdateStatistics()
									end
								end,
					},
				},
			},
			debug = {
				name = L["Debug"],
				type = "group",
				order = 93,
				args = {
					TraceFunctions = {
						name = "Trace function entry/exit",
						order = 1,
						desc = "Output debug text on entry and exit of functions that support it.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceFunctions = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceFunctions
							  end,
					},
					TraceMisc = {
						name = "Trace misc. state",
						order = 2,
						desc = "Output extra debug information scattered throughout.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceMisc = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceMisc
							  end,
					},
					LogValueCalc = {
						name = "Log value calculations",
						order = 3,
						desc = "Record value calculations in log field of SavedVariables.",
						type = "toggle",
						set = function(info, val)
									self.db.profile.debug.LogValueCalc = val
								end,
						get = function(info)
									return self.db.profile.debug.LogValueCalc
								end,
					},
					Calculate = {
						name = "Recalc",
						order = 4,
						desc = "Recalculate values.",
						type = "execute",
						func = function()
									self:CalcValues()
								end,
					},
				}
			},
		}
	}

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Capitalist", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Capitalist")
	self:RegisterChatCommand("Capitalist", function(...) Capitalist:HandleSlashCommand(...) end)
	self:RegisterChatCommand("cap", function(...) Capitalist:HandleSlashCommand(...) end)

	self:DebugExit("BuildOptions")
end

function Capitalist:BuildProfessionFilters(profession)
	local professionData = self.db.profile.professions[profession]
	local filterTerm = self.inclusiveTerm or ""

	-- populate filters
	local t = {}

	for skillName, skillData in pairs(professionData) do
		local product = skillData
		self:DebugMisc(skillName)
		self:DebugMisc(product)
	
		if (self:ProductIsVisible(profession, skillName)) then
			local settingSkillName = L.SettingUIFromSkillName(profession, skillName)
			t[skillName] = {
				name = settingSkillName,
				desc = product.description,
				type = "toggle",
				hidden = (filterTerm and (filterTerm ~= "") and (strlower(settingSkillName):find(strlower(filterTerm)) == nil)),
				set = function(info, val)
						self:ProductSetEnabled(profession, skillName, val)
						self:ApplyFilters()
						end,
				get = function(info)
						return self:ProductIsEnabled(profession, skillName)
						end
			}
		end
	end

	t.overview = {
		name = string.format(L["Profession Filters Overview"], profession),
		type = "description",
		order = 1,
	}
	t.inclusiveTerm = {
		name = L["Search Term"],
		desc = L["Search Term Tooltip"],
		type = "input",
		order = 2,
		width = "full",
		dialogControl = "Confirmless_EditBox",
		set = function(info, val)
					if (self.inclusiveTerm ~= val) then
						self.inclusiveTerm = val

						for profession, professionData in pairs(self.db.profile.professions) do
							self.options.args[profession].args.filters.args = self:BuildProfessionFilters(profession)
						end
						
						AceConfigRegistry:NotifyChange("Capitalist")
					end
				end,
		get = function(info)
					return self.inclusiveTerm
				end
	}
	t.all = {
		name = string.format(L["All %s"], profession),
		desc = string.format(L["All Tooltip"], profession),
		type = "toggle",
		order = 3,
		tristate = true,
		set = function(info, val)
					for skillName, skillData in pairs(professionData) do
						if (self:ProductIsVisible(profession, skillName)) then
							self:ProductSetEnabled(profession, skillName, val)
						end
					end
					self:ApplyFilters()
				end,
		get = function(info)
					local found
					for skillName, skillData in pairs(professionData) do
						if (self:ProductIsVisible(profession, skillName)) then
							local productEnabled = self:ProductIsEnabled(profession, skillName)
							if ((found ~= nil) and (found ~= productEnabled)) then
								return nil
							else
								found = productEnabled
							end
						end
					end
					return found
				end
	}
	t.divider = {
		name = "",
		type = "header",
		order = 4,
	}

	return t
end

function Capitalist:UpdateProductOptions()
	self:DebugEntry("UpdateProductOptions")

	if (self.db.profile.professions) then
		-- add profession nodes
		local order = 11

		local knownProfessions = {}

		if (self.db.profile.professions) then
			for profession, professionData in pairs(self.db.profile.professions) do
				table.insert(knownProfessions, profession)
			end
		end
		table.sort(knownProfessions)

		for i=1, #knownProfessions do
			local profession = knownProfessions[i]
			self.options.args[profession] = {
												name = profession,
												type = "group",
												order = order,
												args = {
													overview = {
														name = string.format(L["Profession Overview"], profession),
														type = "description",
														order = 0,
													},
													filters = {
														name = L["Filters"],
														type = "group",
														order = 1,
														args = {}
													},
													counts = {
														name = L["Counts"],
														type = "group",
														order = 2,
														args = {}
													},
												}
											}
			order = order + 1
		end	

		for profession, professionData in pairs(self.db.profile.professions) do
			if (self.options.args[profession]) then
				self:DebugMisc(profession)

				local t

				self.options.args[profession].args.filters.args = self:BuildProfessionFilters(profession)

				-- populate product count
				self.options.args[profession].args.counts.args = {}
				t = self.options.args[profession].args.counts.args

				for skillName, skillData in pairs(professionData) do
					if (self.db.profile.professions[profession] and
						self.db.profile.professions[profession][skillName]) then

						local product = self.db.profile.professions[profession][skillName]
						self:DebugMisc(skillName)
						self:DebugMisc(product)

						if (self:ProductIsEnabled(profession, skillName) and
							self:ProductIsVisible(profession, skillName)) then

							local settingSkillName = L.SettingUIFromSkillName(profession, skillName)

							t[skillName] = {
								name = settingSkillName,
								desc = product.description,
								type = "input",
								pattern = "^%d+$",
								set = function(info, val)
											local nVal = tonumber(val) or 0
											self:ProductSetCount(profession, skillName, nVal)
										end,
								get = function(info, val)
											return ""..self:ProductGetCount(profession, skillName)
										end
							}
						end
					end
				end

				t.overview = {
					name = string.format(L["Profession Counts Overview"], profession),
					type = "description",
					order = 1,
				}
				t.footnote = {
					name = string.format(L["Profession Counts Footnote"], profession),
					type = "description",
					order = -1,
				}
			end
		end
	end

	self:DebugExit("UpdateProductOptions")
end

function Capitalist:UpdateResourceCosts()
	self:DebugEntry("UpdateResourceCosts")

	local t

	self.options.args.resourceCosts.args = {}
	t = self.options.args.resourceCosts.args

	t.overview = {
		name = L["Resource Costs Overview"],
		type = "description",
		order = 0,
	}
	t.footnote = {
		name = L["Resource Costs Footnote"],
		type = "description",
		order = -1,
	}

	t["BuyPrice"] = {
		name = L["Buy Price"],
		type = "description",
		order = 1,
		width = "half",
	}
	t["AssumedCost"] = {
		name = L["Assumed Cost"],
		type = "description",
		order = 2,
		width = "half",
	}

	-- populate resource costs

	local resourceList = self:GetVisibleResources(true)
	table.sort(resourceList)

	for i = 1, #resourceList do
		local resourceName = resourceList[i]
		t[resourceName.."Header"] = {
			name = resourceName,
			type = "header",
			order = i * 3,
		}
		t[resourceName.."Min"] = {
			name = "",
			type = "input",
			pattern = "^[0-9]*\.?[0-9]+$",
			order = i * 3 + 1,
			width = "half",
			-- TODO: active setting functions
			-- TODO: validate/correct if min/max flip
			set = function(info, val)
						local nVal = (tonumber(val) or 0) * 10000
						self:SetResourceCopperBuyPrice(resourceName, nVal)
						self:CostMemoryReset()
					end,
			get = function(info)
						local nVal = self:GetItemCopperShoppingPrice(resourceName)
						return ""..(nVal / 10000)
					end
		}
		t[resourceName.."Max"] = {
			name = "",
			type = "input",
			pattern = "^[0-9]*\.?[0-9]+$",
			order = i * 3 + 2,
			width = "half",
			-- TODO: active setting functions
			-- TODO: validate/correct if min/max flip
			set = function(info, val)
						local nVal = (tonumber(val) or 0) * 10000
						self:SetResourceCopperCost(resourceName, nVal)
						self:CostMemoryReset()
					end,
			get = function(info)
						return ""..(self:GetResourceCopperCost(resourceName) / 10000)
					end
		}
	end

	self:DebugExit("UpdateResourceCosts")
end

function Capitalist:UpdateStatistics()
	self:DebugEntry("UpdateStatistics")

	if (self.db.profile.professions) then
		local t

		-- populate rare statistics
		self.options.args.rareStatistics.args = {}
		t = self.options.args.rareStatistics.args

		t["overview"] = {
			name = string.format(L["Rare Statistics Overview"], character),
			type = "description",
			order = 1,
		}

		-- populate salvage statistics
		self.options.args.salvageStatistics.args = {}
		t = self.options.args.salvageStatistics.args

		local salvageOrderNext = 1
		t["overview"] = {
			name = string.format(L["Salvage Statistics Overview"], character),
			type = "description",
			order = salvageOrderNext,
		}
		salvageOrderNext = salvageOrderNext + 1
		
		local rareStatisticsArgs = self.options.args.rareStatistics.args
		local salvageStatisticsArgs = self.options.args.salvageStatistics.args

		-- populate created products
		self.options.args.createdProducts.args = {}
		t = self.options.args.createdProducts.args

		local productsTally = {}
		for profession, professionData in pairs(self.db.profile.professions) do

			if (self.options.args[profession]) then
				local oProfession = self:GetProfile_Realm_Faction_Character_Profession(realm, faction, character, profession)
				if (oProfession) then
					for skillName, skillData in pairs(oProfession) do
						local numProduct = skillData.numProduct or 0
						if (professionData[skillName]) then
							local product = professionData[skillName]
							local productText = self:ProductTextFromSkillName(profession, skillName)
							if (productText and (numProduct > 0)) then
								productsTally[productText] = (productsTally[productText] or 0) + numProduct

								if (skillData.rareTally) then
									for rareItemLink, rareItemCount in pairs(skillData.rareTally) do
										local percentage = math.floor(10000 * (rareItemCount / skillData.numCraft)) / 100
										rareStatisticsArgs[rareItemLink] = {
											name = rareItemCount.." x "..rareItemLink.." ("..percentage.."%)",
											type = "description",
											order = -rareItemCount
										}
									end
								end
							end
						end
					end
				end

			end
		end

		t["overview"] = {
			name = string.format(L["Created Products Overview"], character),
			type = "description",
			order = 1,
		}

		-- populate used skills
		self.options.args.usedSkills.args = {}
		t = self.options.args.usedSkills.args

		for profession, professionData in pairs(self.db.profile.professions) do

			if (self.options.args[profession]) then
				local oProfession = self:GetProfile_Realm_Faction_Character_Profession(realm, faction, character, profession)
				if (oProfession) then
					for skillName, skillData in pairs(oProfession) do
						local numCraft = skillData.numCraft or 0
						if (professionData[skillName]) then
							local product = professionData[skillName]
							local productText = self:ProductTextFromSkillName(profession, skillName)
							if (productText and (numCraft > 0)) then
								t[skillName] = {
									name = numCraft.."x "..productText,
									type = "description",
									order = -numCraft,
								}
							end
						end
					end
				end


			end
		end

		t["overview"] = {
			name = string.format(L["Used Skills Overview"], character),
			type = "description",
			order = 1,
		}

		-- REVIEW: Created Products needs a rename now that we track resource production as well.
		-- populate created products and used skills for salvage skills
		local resourcesTally = { }
		local oCharacter = self:GetProfile_Realm_Faction_Character(realm, faction, character)
		for characterSection, characterSectionData in pairs(oCharacter) do
			if (not self.db.profile.professions[characterSection]) then
				-- Assume non-profession sections are salvage skill sections
				local salvageSkill = characterSection
				local salvageSkillData = characterSectionData
				local salvageSkillLink = characterSectionData.recipeLink

				if (salvageSkillLink) then
					salvageStatisticsArgs[salvageSkillLink.."Header"] = {
						name = salvageSkillLink,
						type = "header",
						order = salvageOrderNext,
					}
					salvageOrderNext = salvageOrderNext + 1

					for skillTarget, skillTargetData in pairs(characterSectionData) do
						if ((skillTarget ~= "recipeLink") and
						    (skillTarget ~= "numConsume")) then
							-- Assume non-recipeLink, non-numConsume entries are skillTarget sections
	
							salvageStatisticsArgs[salvageSkillLink.."-"..skillTarget.."Header"] = {
								name = skillTargetData.numCraft.."x "..skillTarget,
								type = "header",
								order = salvageOrderNext,
							}
							salvageOrderNext = salvageOrderNext + 1
	
							local percentageTotal = 0
							for skillTargetResource, skillTargetResourceCount in pairs(skillTargetData.resourceTally) do
								resourcesTally[skillTargetResource] = (resourcesTally[skillTargetResource] or 0) + skillTargetResourceCount;
								local percentage = math.floor(10000 * (skillTargetResourceCount / skillTargetData.numCraft)) / 100
	
								salvageStatisticsArgs[salvageSkillLink.."-"..skillTarget.."-"..skillTargetResource] = {
									name = skillTargetResourceCount.." x "..skillTargetResource.." ("..percentage.."%)",
									type = "description",
									order = salvageOrderNext,
								}
								salvageOrderNext = salvageOrderNext + 1
	
								percentageTotal = percentageTotal + percentage
							end
	
							t = self.options.args.usedSkills.args
							t[salvageSkill..skillTarget] = {
								name = skillTargetData.numCraft.."x "..salvageSkillLink..skillTarget,
								type = "description",
								order = -skillTargetData.numCraft,
							}
						end
					end

					salvageStatisticsArgs[salvageSkillLink.."--"] = {
						name = "",
						type = "header",
						order = salvageOrderNext,
					}
					salvageOrderNext = salvageOrderNext + 1
				end
			end
		end

		t = self.options.args.createdProducts.args
		for resource, resourceCount in pairs(resourcesTally) do
			t[resource] = {
				name = resourceCount.."x "..resource,
				type = "description",
				order = -resourceCount,
			}
		end
		for productText, productCount in pairs(productsTally) do
			t[productText] = {
				name = productCount.."x "..productText,
				type = "description",
				order = -productCount,
			}
		end
	end

	self:DebugExit("UpdateStatistics")
end
